@@ -10,7 +10,12 @@ from typing_extensions import Literal
 
 from pycep import typing as pycep_typing
 
-BICEP_REGISTRY_PATTERN = re.compile(r"br:(?P<registry_name>\w+)\.azurecr\.io/(?P<path>[\w/]+):(?P<tag>[\w.\-]+)")
+BICEP_REGISTRY_ALIAS_PATTERN = re.compile(r"br/(?P<alias>[\w]+):(?P<path>[\w/\-]+):(?P<tag>[\w.\-]+)")
+PUBLIC_BICEP_REGISTRY_PATTERN = re.compile(r"br:mcr\.microsoft\.com/(?P<path>[\w/\-]+):(?P<tag>[\w.\-]+)")
+PRIVATE_BICEP_REGISTRY_PATTERN = re.compile(
+    r"br:(?P<registry_name>\w+)\.azurecr\.io/(?P<path>[\w/\-]+):(?P<tag>[\w.\-]+)"
+)
+TEMPLATE_SPEC_ALIAS_PATTERN = re.compile(r"ts/(?P<alias>[\w]+):(?P<path>[\w/\-]+):(?P<tag>[\w.\-]+)")
 TEMPLATE_SPEC_PATTERN = re.compile(
     r"ts:(?P<sub_id>[\d\-]+)/(?P<rg_id>[\w._\-()]+)/(?P<name>[\w.\-]+):(?P<version>[\w.\-]+)"
 )
@@ -41,7 +46,7 @@ class BicepToJson(Transformer[Token, pycep_typing.BicepJson]):
     def start(self, args: list[pycep_typing.ElementResponse]) -> pycep_typing.BicepJson:
         result: pycep_typing.BicepJson = {
             "globals": {
-                "scope": {  # type: ignore[typeddict-item]
+                "scope": {
                     "value": "resourceGroup",
                 },
             }
@@ -74,7 +79,7 @@ class BicepToJson(Transformer[Token, pycep_typing.BicepJson]):
         result: pycep_typing.ScopeResponse = {
             "globals": {
                 "__name__": "scope",
-                "__attrs__": {  # type: ignore[typeddict-item]
+                "__attrs__": {
                     "value": value,
                 },
             }
@@ -95,7 +100,7 @@ class BicepToJson(Transformer[Token, pycep_typing.BicepJson]):
         result: pycep_typing.ParamResponse = {
             "parameters": {
                 "__name__": str(name),
-                "__attrs__": {  # type: ignore[typeddict-item]
+                "__attrs__": {
                     "decorators": decorators if decorators else [],
                     "type": data_type,
                     "default": default,
@@ -110,13 +115,16 @@ class BicepToJson(Transformer[Token, pycep_typing.BicepJson]):
         return result
 
     @v_args(meta=True)
-    def var(self, meta: Meta, args: tuple[Token, pycep_typing.PossibleValue]) -> pycep_typing.VarResponse:
-        name, value = args
+    def var(
+        self, meta: Meta, args: tuple[list[pycep_typing.Decorator] | None, Token, pycep_typing.PossibleValue]
+    ) -> pycep_typing.VarResponse:
+        decorators, name, value = args
 
         result: pycep_typing.VarResponse = {
             "variables": {
                 "__name__": str(name),
-                "__attrs__": {  # type: ignore[typeddict-item]
+                "__attrs__": {
+                    "decorators": decorators if decorators else [],
                     "value": value,
                 },
             }
@@ -129,13 +137,16 @@ class BicepToJson(Transformer[Token, pycep_typing.BicepJson]):
         return result
 
     @v_args(meta=True)
-    def output(self, meta: Meta, args: tuple[str, str, pycep_typing.PossibleValue]) -> pycep_typing.OutputResponse:
-        name, data_type, value = args
+    def output(
+        self, meta: Meta, args: tuple[list[pycep_typing.Decorator] | None, str, str, pycep_typing.PossibleValue]
+    ) -> pycep_typing.OutputResponse:
+        decorators, name, data_type, value = args
 
         result: pycep_typing.OutputResponse = {
             "outputs": {
                 "__name__": str(name),
-                "__attrs__": {  # type: ignore[typeddict-item]
+                "__attrs__": {
+                    "decorators": decorators if decorators else [],
                     "type": data_type,
                     "value": value,
                 },
@@ -166,7 +177,8 @@ class BicepToJson(Transformer[Token, pycep_typing.BicepJson]):
                 "__name__": name_str,
                 "__attrs__": {
                     "decorators": decorators if decorators else [],
-                    **type_api_pair,  # type: ignore[misc] # https://github.com/python/mypy/issues/11753
+                    "type": type_api_pair["type"],
+                    "api_version": type_api_pair["api_version"],
                     "existing": True if existing else False,
                     "config": config,
                 },
@@ -187,7 +199,7 @@ class BicepToJson(Transformer[Token, pycep_typing.BicepJson]):
 
         result: pycep_typing.Resource = {
             "__name__": str(name),
-            "__attrs__": {  # type: ignore[typeddict-item]
+            "__attrs__": {
                 "decorators": [],
                 "type": str(type_name)[1:-1],
                 "api_version": "",  # will be set later with the parent resource api version
@@ -211,7 +223,7 @@ class BicepToJson(Transformer[Token, pycep_typing.BicepJson]):
 
         result: pycep_typing.ModuleResponse = {
             "modules": {
-                "__name__": name,
+                "__name__": str(name),
                 "__attrs__": {
                     "decorators": decorators if decorators else [],
                     **path,  # type: ignore[misc] # https://github.com/python/mypy/issues/11753
@@ -246,20 +258,54 @@ class BicepToJson(Transformer[Token, pycep_typing.BicepJson]):
         file_path = str(args[0])[1:-1]
 
         if file_path.startswith("br:"):
-            m = re.match(BICEP_REGISTRY_PATTERN, file_path)
-            if not m:  # pragma: no cover
-                raise ValueError(f"Bicep registry path is invalid: {file_path}")
+            # check if it's referencing the official public registry
+            m = re.match(PUBLIC_BICEP_REGISTRY_PATTERN, file_path)
+            if m:
+                is_public = True
+                registry_name = "mcr.microsoft.com"
+            else:
+                # check if it's referencing a privvte registry
+                m = re.match(PRIVATE_BICEP_REGISTRY_PATTERN, file_path)
+                if m:
+                    is_public = False
+                    registry_name = m.group("registry_name")
+                else:  # pragma: no cover
+                    raise ValueError(f"Bicep registry path is invalid: {file_path}")
 
             br_result: pycep_typing.BicepRegistryModulePath = {
                 "type": "bicep_registry",
                 "detail": {
                     "full": file_path[3:],
-                    "registry_name": m.group("registry_name"),
+                    "registry_name": registry_name,
                     "path": m.group("path"),
                     "tag": m.group("tag"),
+                    "public": is_public,
                 },
             }
             return br_result
+        elif file_path.startswith("br/"):
+            m = re.match(BICEP_REGISTRY_ALIAS_PATTERN, file_path)
+            if not m:  # pragma: no cover
+                raise ValueError(f"Bicep registry alias is invalid: {file_path}")
+
+            alias = m.group("alias")
+            is_public = False
+
+            # check if it's the alias of the official public registry
+            if alias == "public":
+                is_public = True
+
+            br_alias_result: pycep_typing.BicepRegistryAliasModulePath = {
+                "type": "bicep_registry_alias",
+                "detail": {
+                    "full": file_path[3:],
+                    "alias": alias,
+                    "path": m.group("path"),
+                    "tag": m.group("tag"),
+                    "public": is_public,
+                },
+            }
+            return br_alias_result
         elif file_path.startswith("ts:"):
             m = re.match(TEMPLATE_SPEC_PATTERN, file_path)
             if not m:  # pragma: no cover
@@ -276,6 +322,21 @@ class BicepToJson(Transformer[Token, pycep_typing.BicepJson]):
                 },
             }
             return ts_result
+        elif file_path.startswith("ts/"):
+            m = re.match(TEMPLATE_SPEC_ALIAS_PATTERN, file_path)
+            if not m:  # pragma: no cover
+                raise ValueError(f"Template spec alias is invalid: {file_path}")
+
+            ts_alias_result: pycep_typing.TemplateSpecAliasModulePath = {
+                "type": "template_spec_alias",
+                "detail": {
+                    "full": file_path[3:],
+                    "alias": m.group("alias"),
+                    "path": m.group("path"),
+                    "tag": m.group("tag"),
+                },
+            }
+            return ts_alias_result
 
         local_result: pycep_typing.LocalModulePath = {
             "type": "local",
@@ -719,14 +780,21 @@ class BicepToJson(Transformer[Token, pycep_typing.BicepJson]):
     #
     ####################
 
-    def json_func(self, args: tuple[pycep_typing.PossibleValue]) -> pycep_typing.Json:
-        return {
+    def json_func(self, args: tuple[pycep_typing.PossibleValue, pycep_typing.PossibleNoneValue]) -> pycep_typing.Json:
+        arg_1, property_name = args
+
+        result: pycep_typing.Json = {
             "type": "json",
             "parameters": {
-                "arg_1": args[0],
+                "arg_1": arg_1,
             },
         }
 
+        if property_name:
+            result["property_name"] = str(property_name)
+
+        return result
+
     ####################
     #
     # functions - resource
